<?php

namespace Drp\UserManagement\Contracts;

use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;

/**
 * @package App\Worldview\Users
 */
interface ManagedUser extends AuthenticatableContract, CanResetPasswordContract
{
    /**
     * Get array of users that meet the given roles.
     *
     * @param string|array $roles
     * @return \Illuminate\Database\Eloquent\Collection
     */
    public function usersByRoles($roles);

    /**
     * Return all the admin users.
     *
     * @return \Illuminate\Database\Eloquent\Collection
     */
    public function allAdmins();

    /**
     * Return all the super admin users.
     *
     * @return \Illuminate\Database\Eloquent\Collection
     */
    public function allSuperAdmins();

    /**
     * Return if the user has admin access.
     *
     * @return boolean
     */
    public function hasAdminAccess();

    /**
     * Determine if the user has super user access.
     *
     * @return boolean
     */
    public function hasSuperAdminAccess();

    /**
     * Determine if the user can manage other admins.
     *
     * @return boolean
     */
    public function canManageAdmins();

    /**
     * Return if the user has the given role.
     *
     * @param string $role The Role to check against
     * @return boolean
     */
    public function hasRole($role);

    /**
     * Finds a user by their one time login token.
     *
     * @param string $token The token to retrieve the user
     * @return mixed
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException If no user was found.
     */
    public function getByFirstLoginToken($token);

    /**
     *  Return all users that are awaiting approval.
     *
     * @return \Illuminate\Support\Collection
     */
    public function allAwaitingApproval();

    /**
     * Return if the user is awaiting approval.
     *
     * @return boolean
     */
    public function requiresApproving();

    /**
     * Approve the user.
     *
     * @return static
     */
    public function approve();

    /**
     * Removes a one time login token from a user.
     *
     * @return static
     */
    public function removeFirstLoginToken();

    /**
     * Store the login token against the user.
     *
     * @param mixed $token
     * @param \DateTime $expiry
     * @return static
     */
    public function addLoginToken($token, \DateTime $expiry);

    /**
     * Return the login token the user can use to access the site.
     *
     * @return string|null
     */
    public function loginToken();

    /**
     * Return the expiry of the login token.
     *
     * @return \DateTime|null
     */
    public function loginExpiry();
}
